<?php
namespace Teknisa\PLA\Modules\Cardapio\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;

class Pla00128ExclusaoCardapio extends \Teknisa\GER\Controllers\TeknisaCrud {
    
    const Card_Bloq = "Existe(m) cardápio(s) com bloqueio(s). Os dias que não podem ser alterados serão desconsiderados.";
    const Delete_Success = "Operação realizada com sucesso.";
    const No_Card = "Não há cardápio no período informado.";
    
    public function __construct($crudService, $parameterBag, $dataSourceManager, $environment, $newCode, $connection, $util) {
        $this->util              = $util;
        $this->newCode           = $newCode;
        $this->connection        = $connection;
        $this->crudService       = $crudService;
        $this->environment       = $environment;
        $this->parameterBag      = $parameterBag;
        $this->dataSourceManager = $dataSourceManager;
    }
    
    public function excluir(DTO\Request $request, DTO\Response $response) {
        set_time_limit(120);
        try {
            $row = $request->getRow();
            $servFili = false;
            if($row['ALLCDSERVFILI'] === 'S') {
                $servicos = $this->getServicos($row['CDGRUPFILI'] ? $row['CDGRUPFILI'] : 'T', $row['ALLCDFILIAL'] === 'S' ? 'T' : join('_', $row['CDFILIAL']));
                // $nrlistaserv = $this->insertListaServicos($servicos);
                $servFili = self::returnServFili($servicos);
                $nrlistaserv = $this->util->insertLista($servFili);
            } else {
                // $nrlistaserv = $this->insertListaServicos($row['FILISERV']);
                $nrlistaserv = $this->util->insertLista($row['FILISERV']);
            }
            $numberServices = $servFili && is_array($servFili) ? sizeof($servFili) : sizeof($row['SERVICOS']);
            $modoProc = $this->getModoProc();
            $tipoCardServ = $this->verificaTipoCardapioPorServico($nrlistaserv, $numberServices);

            if($modoProc === 'NOT_FOUNDED_GROUP_OPER'){ 
                /* Foi definido que quando não há grupo cadastrado
                    para alterar o cardápio, o mesmo deve executar os SELECT's / 
                    operações do Cardápio tipo Livre
                */
                $tipoCardServ = "N";
            }
            $cardapio = array();
            $result = "";
            switch($tipoCardServ) {
                case 'N': // livre
                    $cardapio = $this->prepareCardLivreToDelete($nrlistaserv, $row, $result);
                    break;
                case 'D': // descentralizado
                    $cardapio = $this->prepareCardDescToDelete($modoProc, $nrlistaserv, $row, $result);
                    break;
                case 'S': // padronizado
                    $cardapio = $this->prepareCardPadrToDelete($modoProc, $nrlistaserv, $row, $result);
                    break;
            }

            foreach($cardapio as $rowCardapio) {
                $this->checkCardToDelete($modoProc, $rowCardapio, $tipoCardServ);
                $this->insertLogCard($rowCardapio, $row['DTINTERVAL']['start'], $row['DTINTERVAL']['end']);
                $this->deleteCompCard($rowCardapio);
                $this->deleteCardaDia($rowCardapio);
            }
            $this->deleteLista($nrlistaserv);
            $response->addDataSet(new DataSet('result', array($result)));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }
    
    private function checkCardToDelete($modoProc, $rowCardapio, $tipoCardServ) {
        if($this->dependeLiberacao($tipoCardServ, $rowCardapio['CDFILIAL'], $rowCardapio['CDSERVFILI'])) {
            if($modoProc === 'M') { // matriz
                $cardLib = $this->verificaCardapioLiberado($rowCardapio);
                if($cardLib !== 'N') {
                    $pratos = $this->getPratosEspeCard($cardLib, $rowCardapio['CDFILIAL'], $rowCardapio['NRCARDAPIO']);
                    foreach ($pratos as $prato) {
                        $this->insertCompCardDel($rowCardapio, $prato, 'M');
                    }
                }
            } else { // filial
                $pratos = $this->getPratosCard($rowCardapio['CDFILIAL'], $rowCardapio['NRCARDAPIO']);
                foreach ($pratos as $prato) {
                    $this->insertCompCardDel($rowCardapio, $prato, 'F');
                }
            }
        }
        if($this->verificaTipoCardapioPorFilial($rowCardapio['CDFILIAL']) === 'S') {
            $rowCentCmp = getCentralCompras($rowCardapio['CDFILIAL']);
            if($rowCentCmp && $rowCentCmp['CDFILCENTPME']) {
                $pratos = $this->getPratosCard($rowCardapio['CDFILIAL'], $rowCardapio['NRCARDAPIO']);
                foreach ($pratos as $prato) {
                    if(sizeof($this->getCompCardDel($rowCardapio['CDFILIAL'], $rowCardapio['CDSERVFILI'], $rowCardapio['DTCARDAPIO'], $prato['NRSEQPRATCAR']))) {
                        $this->updateCompCardDel($rowCardapio, $prato, $modoProc);
                    } else {
                        $this->insertCompCardDel($rowCardapio, $prato, $modoProc);
                    }
                }
            }
        }
    }
    
    private function prepareCardDescToDelete($modoProc, $nrlistaserv, $row, &$result) {
        $cardapio = array();
        if(sizeof($this->verificaCardadia($row['DTINTERVAL']['start'], $row['DTINTERVAL']['end'], $nrlistaserv))) {
            if($modoProc === 'M') { // matriz
                $cardapio = $this->getCardapioDescMatr($row['DTINTERVAL']['start'], $row['DTINTERVAL']['end'], $nrlistaserv);
                if(sizeof($this->getCardapioBloqDescMatr($row['DTINTERVAL']['start'], $row['DTINTERVAL']['end'], $nrlistaserv))) {
                    $result = $this->getCardapioBloqDescMatr($row['DTINTERVAL']['start'], $row['DTINTERVAL']['end'], $nrlistaserv);
                } else {
                    $result = 'Delete_Success';
                }
            } else { // filial
                $cardapio = $this->getCardapioDescFili($row['DTINTERVAL']['start'], $row['DTINTERVAL']['end'], $nrlistaserv);
                if(sizeof($this->getCardapioBloqDescFili($row['DTINTERVAL']['start'], $row['DTINTERVAL']['end'], $nrlistaserv))) {
                    $result = $this->getCardapioBloqDescFili($row['DTINTERVAL']['start'], $row['DTINTERVAL']['end'], $nrlistaserv);
                } else {
                    $result = 'Delete_Success';
                }
            }
        } else {
            $result = 'No_Card';
        }
        return $cardapio;
    }
    
    private function prepareCardLivreToDelete($nrlistaserv, $row, &$result) {
        $cardapio = array();
        if(sizeof($this->verificaCardadia($row['DTINTERVAL']['start'], $row['DTINTERVAL']['end'], $nrlistaserv))) {
            $cardapio = $this->getCardapio($row['DTINTERVAL']['start'], $row['DTINTERVAL']['end'], $nrlistaserv);
            if(sizeof($this->getCardapioBloq($row['DTINTERVAL']['start'], $row['DTINTERVAL']['end'], $nrlistaserv))) {
                $result = $this->getCardapioBloq($row['DTINTERVAL']['start'], $row['DTINTERVAL']['end'], $nrlistaserv);
            } else {
                $result = 'Delete_Success';
            }
        } else {
            $result = 'No_Card';
        }
        return $cardapio;
    }
    
    private function prepareCardPadrToDelete($modoProc, $nrlistaserv, $row, &$result) {
        $cardapio = array();
        if(sizeof($this->verificaCardadia($row['DTINTERVAL']['start'], $row['DTINTERVAL']['end'], $nrlistaserv))) {
            if($modoProc === 'M') { // matriz
                $cardapio = $this->getCardapioPadrMatr($row['DTINTERVAL']['start'], $row['DTINTERVAL']['end'], $nrlistaserv);
                if(sizeof($this->getCardapioBloqPadrMatr($row['DTINTERVAL']['start'], $row['DTINTERVAL']['end'], $nrlistaserv))) {
                    $result = $this->getCardapioBloqPadrMatr($row['DTINTERVAL']['start'], $row['DTINTERVAL']['end'], $nrlistaserv);
                } else {
                    $result = 'Delete_Success';
                }
            } else { // filial
                $cardapio = $this->getCardapioPadrFili($row['DTINTERVAL']['start'], $row['DTINTERVAL']['end'], $nrlistaserv);
                if(sizeof($this->getCardapioBloqPadrFili($row['DTINTERVAL']['start'], $row['DTINTERVAL']['end'], $nrlistaserv))) {
                    $result = $this->getCardapioBloqPadrFili($row['DTINTERVAL']['start'], $row['DTINTERVAL']['end'], $nrlistaserv);
                } else {
                    $result = 'Delete_Success';
                }
            }
        } else {
            $result = 'No_Card';
        }
        return $cardapio;
    }
    
    private function deleteCardaDia($cardapio) {
        $query = "DELETE FROM 
                    CARDADIA
                WHERE
                    CDFILIAL = :CDFILIAL AND
                    INSTR(:NRCARDAPIO, NRCARDAPIO) <> 0";
        $params = array(
            "CDFILIAL" => $cardapio['CDFILIAL'],
            "NRCARDAPIO" => $cardapio['NRCARDAPIO']
        );
        $this->connection->prepare($query)->execute($params);
    }
    
    private function deleteCompCard($cardapio) {
        $query = "DELETE FROM 
                    COMPCARD
                WHERE
                    CDFILIAL = :CDFILIAL AND
                    INSTR(:NRCARDAPIO, NRCARDAPIO) <> 0";
        $params = array(
            "CDFILIAL" => $cardapio['CDFILIAL'],
            "NRCARDAPIO" => $cardapio['NRCARDAPIO']
        );
        $this->connection->prepare($query)->execute($params);
    }
    
    private function dependeLiberacao($tipoCard, $cdfilial, $cdservfili) {
        if($tipoCard === 'D') {
            $query = "SELECT
                        NVL(IDCARDDESCENTR, 'N') AS IDCARDDESCENTR
                    FROM 
                        SERVICO
                    WHERE
                        CDFILIAL = :CDFILIAL AND
                        CDSERVFILI = :CDSERVFILI";
            $params = array(
                "CDFILIAL" => $cdfilial,
                "CDSERVFILI" => $cdservfili
            );
            $result = $this->connection->fetchAll($query, $params);
            return $result[0]['IDCARDDESCENTR'] === "S";
        } else {
            return false;
        }
    }
    
    private function deleteLista($nrlista) {
        $conditions = array(
            array(
                "columnName" => "NRLISTA",
                "operator"   => '=',
                "value"      => $nrlista
            )
        );
        $dataSet = $this->crudService->getFilteredDataSet("lista", $conditions);
        $this->dataSourceManager->delete($dataSet);
    }
    
    private function getCentralCompras($cdfilial) {
        $query = "SELECT
                    CDFILCENTPME
                FROM
                    FILIAL
                WHERE
                    CDFILIAL = :CDFILIAL";
        $params = array(
            "CDFILIAL" => $cdfilial
        );
        return $this->connection->fetchAll($query, $params)[0];
    }
    
    private function getCompCardDel($cdfilial, $cdservfili, $dtpratexcl, $nrseqpratexcl) {
        $query = "SELECT
                    CDFILIAL
                FROM
                    COMPCARDDEL
                WHERE
                    CDFILIAL = :CDFILIAL AND
                    CDSERVFILI = :CDSERVFILI AND
                    DTPRATEXCL = :DTPRATEXCL AND
                    NRSEQPRATEXCL = :NRSEQPRATEXCL";
        $params = array(
            "CDFILIAL" => $cdfilial,
            "CDSERVFILI" => $cdservfili,
            "DTPRATEXCL" => $dtpratexcl,
            "NRSEQPRATEXCL" => $nrseqpratexcl
        );
        return $this->connection->fetchAll($query, $params);
    }
    
    private function getModoProc() {
        $filterCriteria = new FilterCriteria('opergrupop');
        $filterCriteria->addCondition('CDOPERADOR', $this->environment->getCdOperador());
        $filterCriteria->addCondition('CDGRUPOPER', $this->getGrupOper());
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $rows = $dataSet->getRows();
        return $this->getGrupOper() ? (isset($rows[0]) ? 'M' : 'F') : 'NOT_FOUNDED_GROUP_OPER';
    }
    
    private function getGrupOper() {
        $filterCriteria = new FilterCriteria('autgrupop');
        $filterCriteria->addCondition('IDAUTGRUPOP', 'LRCD');
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $rows = $dataSet->getRows();
        return isset($rows[0]) ? $rows[0]['CDGRUPOPER'] : null;
    }
    
    private function getPratosCard($cdfilial, $nrcardapio) {
        $query = "SELECT
                    NRSEQPRATCAR,
                    IDPRATOESPE,
                    CDFILAUXPRAT,
                    CDPRATO,
                    CDFILAUXPRATMAT,
                    CDPRATOMAT
                FROM
                    COMPCARD
                WHERE
                    CDFILIAL =:CDFILIAL AND
                    NRCARDAPIO =:NRCARDAPIO";
        $params = array(
            "CDFILIAL" => $cdfilial,
            "NRCARDAPIO" => $nrcardapio
        );
        return $this->connection->fetchAll($query, $params);
    }
    
    private function getPratosEspeCard($cardLib, $cdfilial, $nrcardapio) {
        $query = "SELECT
                    NRSEQPRATCAR,
                    IDPRATOESPE,
                    CDFILAUXPRAT,
                    CDPRATO,
                    CDFILAUXPRATMAT,
                    CDPRATOMAT
                FROM
                    COMPCARD
                WHERE
                    IDPRATOESPE <>:IDPRATOESPE AND
                    CDFILIAL =:CDFILIAL AND
                    NRCARDAPIO =:NRCARDAPIO";
        $params = array(
            "CDFILIAL" => $cdfilial,
            "IDPRATOESPE" => $cardLib,
            "NRCARDAPIO" => $nrcardapio
        );
        return $this->connection->fetchAll($query, $params);
    }
    
    private function getServicos($cdgrupfili, $cdfilial) {
        $filterCriteria = new FilterCriteria('servicos_por_agrupfili_multi_filiais');
        $this->parameterBagColumns = array('CDGRUPFILI', 'CDFILIAL', 'CDOPERADOR', 'NRORG');
        $filterCriteria->addCondition('CDGRUPFILI', $cdgrupfili);
        $filterCriteria->addCondition('CDFILIAL', $cdfilial);
        $filterCriteria->addCondition('CDOPERADOR', $this->environment->getCdOperador());
        $filterCriteria->addCondition('NRORG', $this->environment->getNrOrg());
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
        return $dataSet->getRows();
    }
    
    private function insertCompCardDel($cardapio, $prato, $idlocalexcl) {
        $query = "INSERT INTO COMPCARDDEL (
                    CDFILIAL,
                    CDSERVFILI,
                    DTPRATEXCL,
                    NRSEQPRATEXCL,
                    IDLOCALEXCL,
                    CDFILAUXPRAT,
                    CDPRATO
                ) VALUES (
                    :CDFILIAL,
                    :CDSERVFILI,
                    :DTPRATEXCL,
                    :NRSEQPRATEXCL,
                    :IDLOCALEXCL,
                    :CDFILAUXPRAT,
                    :CDPRATO
                )";
        if(!sizeof($prato['CDFILAUXPRATMAT']) && !sizeof($prato['CDPRATOMAT'])) {
            $params = array(
                'CDFILIAL' => $cardapio['CDFILIAL'],
                'CDSERVFILI' => $cardapio['CDSERVFILI'],
                'DTPRATEXCL' => $cardapio['DTCARDAPIO'],
                'NRSEQPRATEXCL' => $prato['NRSEQPRATCAR'],
                'IDLOCALEXCL' => $idlocalexcl,
                'CDFILAUXPRAT' => $prato['CDFILAUXPRAT'],
                'CDPRATO' => $prato['CDPRATO']
            );
        } else {
            $params = array(
                'CDFILIAL' => $cardapio['CDFILIAL'],
                'CDSERVFILI' => $cardapio['CDSERVFILI'],
                'DTPRATEXCL' => $cardapio['DTCARDAPIO'],
                'NRSEQPRATEXCL' => $prato['NRSEQPRATCAR'],
                'IDLOCALEXCL' => $idlocalexcl,
                'CDFILAUXPRAT' => $prato['CDFILAUXPRATMAT'],
                'CDPRATO' => $prato['CDPRATOMAT']
            );
        }
        $this->connection->prepare($query)->execute($params);
    }
    
    private function insertListaServicos($servicosArray) {
        $nrlista = $this->newCode->getCodeByTableName("LISTA", $this->environment->getNrOrg());
        $rows = array();
        foreach ($servicosArray as $servico) {
            $row = array(
                "NRLISTA" => $nrlista,
                "CDLISTA" => gettype($servico) === "array" ? $servico['FILISERV'] : $servico,
                "__is_new" => true
            );
            array_push($rows, $row);
        }
        $dataSet = new DataSet("lista", $rows);
        $request = new DTO\Request\DataSet($dataSet, "", "", "");
        $rowsToSave = $this->crudService->getRowsToSave($request, "lista");
        $this->crudService->save("lista", $rowsToSave);
        return $nrlista;
    }
    
    private function insertLogCard($cardapio, $dtini, $dtfim) {
        $query = "INSERT INTO LOGEXCLCARDAPIO (
                    CDFILIAL,
                    CDSERVFILI,
                    CDOPERADOR,
                    DTCARDEXCLU,
                    DTEXCCARDAPIO,
                    DTINICARDAPIO,
                    DTFINCARDAPIO
                ) VALUES (
                    :CDFILIAL,
                    :CDSERVFILI,
                    :CDOPERADOR,
                    :DTCARDEXCLU,
                    SYSDATE,
                    :DTINICARDAPIO,
                    :DTFINCARDAPIO
                )";
        $params = array(
            "CDFILIAL" => $cardapio['CDFILIAL'],
            "CDSERVFILI" => $cardapio['CDSERVFILI'],
            "CDOPERADOR" => $this->environment->getCdOperador(),
            "DTCARDEXCLU" => $cardapio['DTCARDAPIO'],
            "DTINICARDAPIO" => $dtini,
            "DTFINCARDAPIO" => $dtfim
        );
        $this->connection->prepare($query)->execute($params);
    }
    
    private function updateCompCardDel($cardapio, $prato, $idlocalexcl) {
        $query = "UPDATE
                    COMPCARDDEL
                SET
                    IDLOCALEXCL = :IDLOCALEXCL
                WHERE
                    CDFILIAL = :CDFILIAL AND
                    CDSERVFILI = :CDSERVFILI AND
                    DTPRATEXCL = :DTPRATEXCL AND
                    NRSEQPRATEXCL = :NRSEQPRATEXCL";
        $params = array(
            'IDLOCALEXCL' => $idlocalexcl,
            'CDFILIAL' => $cardapio['CDFILIAL'],
            'CDSERVFILI' => $cardapio['CDSERVFILI'],
            'DTPRATEXCL' => $cardapio['DTCARDAPIO'],
            'NRSEQPRATEXCL' => $prato['NRSEQPRATCAR']
        );
        $this->connection->prepare($query)->execute($params);
    }
    
    private function verificaCardadia($dtini, $dtfim, $nrlistaserv) {
        $query = "SELECT
                    NRCARDAPIO
                FROM
                    CARDADIA CA,
                    LISTA LS
                WHERE
                    LS.NRLISTA = :NRLISTASERV AND
                    CA.CDFILIAL = SUBSTR(LS.CDLISTA, 1, 4) AND
                    CA.CDSERVFILI = SUBSTR(LS.CDLISTA, 5, 5) AND
                    CA.DTCARDAPIO BETWEEN :DTINICIAL AND :DTFINAL";
        $params = array(
            "NRLISTASERV" => $nrlistaserv,
            "DTINICIAL" => $dtini,
            "DTFINAL" => $dtfim
        );
        return $this->connection->fetchAll($query, $params);
    }
    
    private function verificaCardapioLiberado($row) {
        $query = "SELECT
                    CDFILIAL,
                    DTLIBECARDMATR
                FROM
                    CTRLAPROVCARD
                WHERE
                    CDFILIAL =:CDFILIAL AND
                    CDSERVFILI =:CDSERVFILI AND
                    DTMESCARDAPROV =:DTMESCARDAPROV AND
                    DTLIBECARDMATR IS NOT NULL";
        $dtmescardaprov = date('01/m/Y', strtotime(str_replace('/', '-', $row['DTCARDAPIO'])));
        $params = array(
            "CDFILIAL" => $row['CDFILIAL'],
            "CDSERVFILI" => $row['CDSERVFILI'],
            "DTMESCARDAPROV" => $row['DTCARDAPIO']
        );
        $result = $this->connection->fetchAll($query, $params);
        if(sizeof($result)) {
            return 'TODOS';
        } else {
            return 'N';
        }
    }
    
    private function verificaTipoCardapioPorFilial($cdfilial) {
        $query = "SELECT
                    S.CDSERVFILI
                FROM
                    SERVICO S
                WHERE
                    S.CDFILIAL = :CDFILIAL AND
                    S.IDCARDDESCENTR = 'S'";
        $params = array(
            "CDFILIAL" => $cdfilial
        );
        $servicos = $this->connection->fetchAll($query, $params);
        $query = "SELECT
                    NVL(IDUTCARDPADR, 'N') AS IDUTCARDPADR
                FROM
                    PARAMGERAL";
        $idutcardpadr = $this->connection->fetchAll($query, [])[0]['IDUTCARDPADR'];
        if($idutcardpadr === 'S' && sizeof($servicos)) {
            return 'D';
        } else {
            return $idutcardpadr;
        }
    }
    
    private function verificaTipoCardapioPorServico($nrlista, $count) {
        $allServ = array();
        for($i = 0 ; $i < ($count/500)+1 ; $i++){
            $y = 500*($i+1);
            $rowNum = "ROWNUM > ".($i*500)." AND ROWNUM <= ".$y." AND ";
            $query = "SELECT 
                        S.CDSERVFILI
                    FROM 
                        SERVICO S,
                        LISTA LS
                    WHERE
                        ".$rowNum
                        ."LS.NRLISTA = :NRLISTASERV AND
                        S.CDFILIAL = SUBSTR(LS.CDLISTA, 1, 4) AND
                        S.IDCARDDESCENTR = 'S'";
            $params = array(
                "NRLISTASERV" => $nrlista
            );
            $servicos = $this->connection->fetchAll($query, $params);
            
            if(!empty($servicos))
                $allServ[] = $servicos;
        }
        $query = "SELECT
                    NVL(IDUTCARDPADR, 'N') AS IDUTCARDPADR
                FROM
                    PARAMGERAL";
        $idutcardpadr = $this->connection->fetchAll($query, [])[0]['IDUTCARDPADR'];
        if($idutcardpadr === 'S' && sizeof($allServ)) {
            return 'D';
        } else {
            return $idutcardpadr;
        }
    }
    
    private function getCardapio($dtini, $dtfim, $nrlistaserv) {
        $query = "SELECT
                    CA.CDFILIAL,
                    CA.NRCARDAPIO,
                    CA.CDSERVFILI,
                    SYSDATE,
                    CA.DTCARDAPIO
                FROM
                    CARDADIA CA,
                    PARAMFILIAL PF,
                    PARAMGERAL PG,
                    LISTA LS
                WHERE
                    LS.NRLISTA = :NRLISTASERV AND
                    CA.CDFILIAL = SUBSTR(LS.CDLISTA, 1, 4) AND
                    CA.CDSERVFILI = SUBSTR(LS.CDLISTA, 5, 5) AND
                    CDDIETPAD IS NULL AND
                    CA.DTCARDAPIO BETWEEN :DTINICIAL AND :DTFINAL AND
                    PF.CDFILIAL = CA.CDFILIAL AND
                    CA.DTCARDAPIO >= TRUNC(DECODE(
                        NVL(PG.IDUTILDTPROC, 'N'),
                        'N',
                        SYSDATE,
                        NVL(PF.DTPROCESSA, SYSDATE)) + NVL(PG.QTDIABLOQCAR, 0)) AND
                    NOT EXISTS (
                        SELECT
                            RE.CDFILIAL
                        FROM
                            RETIRADA RE
                        WHERE
                            RE.CDFILIAL = CA.CDFILIAL AND
                            RE.NRCARDAPIO = CA.NRCARDAPIO) AND
                    NOT EXISTS (
                        SELECT
                            EF.CDFILIAL
                        FROM
                            EFETCLIE EF
                        WHERE
                            EF.CDFILIAL = CA.CDFILIAL AND
                            EF.CDSERVPLAN = CA.CDSERVFILI AND
                            EF.DTEFETCLIE = CA.DTCARDAPIO AND
                            EF.CDFILIFATU IS NOT NULL AND
                            EF.NRAPURACFAT IS NOT NULL
                    ) AND
                    NOT EXISTS (
                        SELECT
                            DC.NRDISTCARD
                        FROM
                            DISTCARD DC
                        WHERE
                            DC.CDFILIAL = CA.CDFILIAL AND
                            DC.NRCARDAPIO = CA.NRCARDAPIO)";
        $params = array(
            "NRLISTASERV" => $nrlistaserv,
            "DTINICIAL" => $dtini,
            "DTFINAL" => $dtfim
        );
        return $this->connection->fetchAll($query, $params);
    }
    
    private function getCardapioBloq($dtini, $dtfim, $nrlistaserv) {
        $query = "SELECT
                    CA.CDFILIAL,
                    CA.NRCARDAPIO,
                    CA.CDSERVFILI,
                    SYSDATE,
                    CA.DTCARDAPIO
                FROM
                    CARDADIA CA,
                    LISTA LS
                WHERE
                        LS.NRLISTA =:NRLISTASERV AND
                        CA.CDFILIAL = SUBSTR(LS.CDLISTA, 1, 4) AND
                        CA.CDSERVFILI = SUBSTR(LS.CDLISTA, 5, 5) AND
                        CDDIETPAD IS NULL AND
                        CA.DTCARDAPIO BETWEEN :DTINICIAL AND :DTFINAL
                        
                MINUS
                
                SELECT
                    CA.CDFILIAL,
                    CA.NRCARDAPIO,
                    CA.CDSERVFILI,
                    SYSDATE,
                    CA.DTCARDAPIO
                FROM
                    CARDADIA CA,
                    PARAMFILIAL PF,
                    PARAMGERAL PG,
                    LISTA LS
                WHERE
                    LS.NRLISTA =:NRLISTASERV AND
                    CA.CDFILIAL = SUBSTR(LS.CDLISTA, 1, 4) AND
                    CA.CDSERVFILI = SUBSTR(LS.CDLISTA, 5, 5) AND
                    CDDIETPAD IS NULL AND
                    CA.DTCARDAPIO BETWEEN :DTINICIAL AND :DTFINAL AND
                    PF.CDFILIAL = CA.CDFILIAL AND
                    CA.DTCARDAPIO >= DECODE(
                        NVL(PG.IDUTILDTPROC, 'N'),
                        'N',
                        SYSDATE,
                        NVL(PF.DTPROCESSA, SYSDATE)) + NVL(PG.QTDIABLOQCAR, 0) AND
                    NOT EXISTS (
                        SELECT
                            RE.CDFILIAL
                        FROM
                            RETIRADA RE
                        WHERE
                            RE.CDFILIAL = CA.CDFILIAL AND
                            RE.NRCARDAPIO = CA.NRCARDAPIO
                    ) AND
                    NOT EXISTS (
                        SELECT
                            EF.CDFILIAL
                        FROM
                            EFETCLIE EF
                        WHERE
                            EF.CDFILIAL = CA.CDFILIAL AND
                            EF.CDSERVPLAN = CA.CDSERVFILI AND
                            EF.DTEFETCLIE = CA.DTCARDAPIO AND
                            EF.CDFILIFATU IS NOT NULL AND
                            EF.NRAPURACFAT IS NOT NULL
                    ) AND
                    NOT EXISTS (
                        SELECT
                            DC.NRDISTCARD
                        FROM
                            DISTCARD DC
                        WHERE
                            DC.CDFILIAL = CA.CDFILIAL AND
                            DC.NRCARDAPIO = CA.NRCARDAPIO)";
        $params = array(
            "NRLISTASERV" => $nrlistaserv,
            "DTINICIAL" => $dtini,
            "DTFINAL" => $dtfim
        );
        return $this->connection->fetchAll($query, $params);
    }
    
    private function getCardapioDescMatr($dtini, $dtfim, $nrlistaserv) {
        $query = "SELECT
                    CA.CDFILIAL,
                    CA.NRCARDAPIO,
                    CA.CDSERVFILI,
                    SYSDATE,
                    CA.DTCARDAPIO
                FROM
                    CARDADIA CA,
                    PARAMFILIAL PF,
                    PARAMGERAL PG,
                    LISTA LS,
                    CTRLAPROVCARD CT
                WHERE
                    LS.NRLISTA =:NRLISTASERV AND
                    CA.CDFILIAL = SUBSTR(LS.CDLISTA, 1, 4) AND
                    CA.CDSERVFILI = SUBSTR(LS.CDLISTA, 5, 5) AND
                    CDDIETPAD IS NULL AND
                    CA.DTCARDAPIO BETWEEN :DTINICIAL AND :DTFINAL AND
                    PF.CDFILIAL = CA.CDFILIAL AND
                    CT.CDFILIAL (+) = CA.CDFILIAL AND
                    CT.CDSERVFILI (+) = CA.CDSERVFILI AND
                    CT.DTMESCARDAPROV (+) = CA.DTMESCARD AND (
                        (CT.DTLIBECARDMATR IS NULL) OR
                        NOT (CT.DTAPROCARDFILI IS NULL)
                    ) AND
                    CA.DTCARDAPIO >= TRUNC(DECODE(
                        NVL(PG.IDUTILDTPROC, 'N'),
                        'N',
                        SYSDATE,
                        NVL(PF.DTPROCESSA, SYSDATE)) + NVL(PG.QTDIABLOQCAR, 0)) AND
                    NOT EXISTS (
                        SELECT
                            RE.CDFILIAL
                        FROM
                            RETIRADA RE
                        WHERE
                            RE.CDFILIAL = CA.CDFILIAL AND
                            RE.NRCARDAPIO = CA.NRCARDAPIO
                    ) AND
                    NOT EXISTS (
                        SELECT
                            EF.CDFILIAL
                        FROM
                            EFETCLIE EF
                        WHERE
                            EF.CDFILIAL = CA.CDFILIAL AND
                            EF.CDSERVPLAN = CA.CDSERVFILI AND
                            EF.DTEFETCLIE = CA.DTCARDAPIO AND
                            EF.CDFILIFATU IS NOT NULL AND
                            EF.NRAPURACFAT IS NOT NULL
                    ) AND
                    NOT EXISTS (
                        SELECT
                            DC.NRDISTCARD
                        FROM
                            DISTCARD DC
                        WHERE
                            DC.CDFILIAL = CA.CDFILIAL AND
                            DC.NRCARDAPIO = CA.NRCARDAPIO)";
        $params = array(
            "NRLISTASERV" => $nrlistaserv,
            "DTINICIAL" => $dtini,
            "DTFINAL" => $dtfim
        );
        return $this->connection->fetchAll($query, $params);
    }
    
    private function getCardapioBloqDescMatr($dtini, $dtfim, $nrlistaserv) {
        $query = "SELECT
                    CA.CDFILIAL,
                    CA.NRCARDAPIO,
                    CA.CDSERVFILI,
                    SYSDATE,
                    CA.DTCARDAPIO
                FROM
                    CARDADIA CA,
                    LISTA LS
                WHERE
                    LS.NRLISTA =:NRLISTASERV AND
                    CA.CDFILIAL = SUBSTR(LS.CDLISTA, 1, 4) AND
                    CA.CDSERVFILI = SUBSTR(LS.CDLISTA, 5, 5) AND
                    CDDIETPAD IS NULL AND
                    CA.DTCARDAPIO BETWEEN :DTINICIAL AND :DTFINAL
                MINUS
                SELECT
                    CA.CDFILIAL,
                    CA.NRCARDAPIO,
                    CA.CDSERVFILI,
                    SYSDATE,
                    CA.DTCARDAPIO
                FROM
                    CARDADIA CA,
                    PARAMFILIAL PF,
                    PARAMGERAL PG,
                    LISTA LS,
                    CTRLAPROVCARD CT
                WHERE
                    LS.NRLISTA =:NRLISTASERV AND
                    CA.CDFILIAL = SUBSTR(LS.CDLISTA, 1, 4) AND
                    CA.CDSERVFILI = SUBSTR(LS.CDLISTA, 5, 5) AND
                    CDDIETPAD IS NULL AND
                    CA.DTCARDAPIO BETWEEN :DTINICIAL AND :DTFINAL AND
                    PF.CDFILIAL = CA.CDFILIAL AND
                    CT.CDFILIAL (+) = CA.CDFILIAL AND
                    CT.CDSERVFILI (+) = CA.CDSERVFILI AND
                    CT.DTMESCARDAPROV (+) = CA.DTMESCARD AND
                    (
                        CT.DTLIBECARDMATR IS NULL OR
                        NOT (CT.DTAPROCARDFILI IS NULL)
                    ) AND
                    CA.DTCARDAPIO >= TRUNC(DECODE(
                        NVL(PG.IDUTILDTPROC, 'N'),
                        'N',
                        SYSDATE,
                        NVL(PF.DTPROCESSA, SYSDATE)) + NVL(PG.QTDIABLOQCAR, 0)) AND
                    NOT EXISTS (
                        SELECT
                            RE.CDFILIAL
                        FROM
                            RETIRADA RE
                        WHERE
                            RE.CDFILIAL = CA.CDFILIAL AND
                            RE.NRCARDAPIO = CA.NRCARDAPIO
                    ) AND
                    NOT EXISTS (
                        SELECT
                            EF.CDFILIAL
                        FROM
                            EFETCLIE EF
                        WHERE
                            EF.CDFILIAL = CA.CDFILIAL AND
                            EF.CDSERVPLAN = CA.CDSERVFILI AND
                            EF.DTEFETCLIE = CA.DTCARDAPIO AND 
                            EF.CDFILIFATU IS NOT NULL AND
                            EF.NRAPURACFAT IS NOT NULL
                    ) AND
                    NOT EXISTS (
                        SELECT
                            DC.NRDISTCARD
                        FROM
                            DISTCARD DC
                        WHERE
                            DC.CDFILIAL = CA.CDFILIAL AND
                            DC.NRCARDAPIO = CA.NRCARDAPIO)";
        $params = array(
            "NRLISTASERV" => $nrlistaserv,
            "DTINICIAL" => $dtini,
            "DTFINAL" => $dtfim
        );
        return $this->connection->fetchAll($query, $params);
    }
    
    private function getCardapioDescFili($dtini, $dtfim, $nrlistaserv) {
        $query = "SELECT
                    CA.CDFILIAL,
                    CA.NRCARDAPIO,
                    CA.CDSERVFILI,
                    SYSDATE,
                    CA.DTCARDAPIO
                FROM
                    CARDADIA CA,
                    PARAMFILIAL PF,
                    PARAMGERAL PG,
                    LISTA LS,
                    CTRLAPROVCARD CT,
                    SERVICO SE
                WHERE
                    LS.NRLISTA =:NRLISTASERV AND
                    CA.CDFILIAL = SUBSTR(LS.CDLISTA, 1, 4) AND
                    CA.CDSERVFILI = SUBSTR(LS.CDLISTA, 5, 5) AND
                    CDDIETPAD IS NULL AND
                    CA.DTCARDAPIO BETWEEN :DTINICIAL AND :DTFINAL AND
                    PF.CDFILIAL = CA.CDFILIAL AND
                    CT.CDFILIAL (+) = CA.CDFILIAL AND
                    CA.CDFILIAL = SE.CDFILIAL AND
                    CA.CDSERVFILI = SE.CDSERVFILI AND
                    SE.IDCARDDESCENTR = 'S' AND
                    CT.CDSERVFILI (+) = CA.CDSERVFILI AND
                    CT.DTMESCARDAPROV (+) = CA.DTMESCARD AND
                    (
                        (CT.DTLIBECARDMATR IS NULL) OR
                        NOT(CT.DTAPROCARDFILI IS NULL)
                    ) AND
                    CA.DTCARDAPIO >= TRUNC(DECODE(
                        NVL(PG.IDUTILDTPROC, 'N'),
                        'N',
                        SYSDATE,
                        NVL(PF.DTPROCESSA, SYSDATE)) + NVL(PG.QTDIABLOQCAR, 0)) AND
                    NOT EXISTS (
                        SELECT
                            RE.CDFILIAL
                        FROM
                            RETIRADA RE
                        WHERE
                            RE.CDFILIAL = CA.CDFILIAL AND
                            RE.NRCARDAPIO = CA.NRCARDAPIO
                    ) AND
                    NOT EXISTS (
                        SELECT
                            EF.CDFILIAL
                        FROM
                            EFETCLIE EF
                        WHERE
                            EF.CDFILIAL = CA.CDFILIAL AND
                            EF.CDSERVPLAN = CA.CDSERVFILI AND
                            EF.DTEFETCLIE = CA.DTCARDAPIO AND
                            EF.CDFILIFATU IS NOT NULL AND
                            EF.NRAPURACFAT IS NOT NULL
                    ) AND
                    NOT EXISTS (
                        SELECT
                            DC.NRDISTCARD
                        FROM
                            DISTCARD DC
                        WHERE
                            DC.CDFILIAL = CA.CDFILIAL AND
                            DC.NRCARDAPIO = CA.NRCARDAPIO)
                UNION ALL
                SELECT
                    CA.CDFILIAL,
                    CA.NRCARDAPIO,
                    CA.CDSERVFILI,
                    SYSDATE,
                    CA.DTCARDAPIO
                FROM
                    CARDADIA CA,
                    PARAMFILIAL PF,
                    PARAMGERAL PG,
                    LISTA LS,
                    SERVICO SE
                WHERE
                    LS.NRLISTA =:NRLISTASERV AND
                    CA.CDFILIAL = SUBSTR(LS.CDLISTA, 1, 4) AND
                    CA.CDSERVFILI = SUBSTR(LS.CDLISTA, 5, 5) AND
                    CDDIETPAD IS NULL AND
                    PG.IDPEALTCARDSERV = 'S' AND
                    CA.DTCARDAPIO BETWEEN :DTINICIAL AND :DTFINAL AND
                    CA.CDFILIAL = SE.CDFILIAL AND
                    CA.CDSERVFILI = SE.CDSERVFILI AND
                    NVL(SE.IDCARDDESCENTR, 'N') = 'N' AND
                    PF.CDFILIAL = CA.CDFILIAL AND
                    CA.DTCARDAPIO >= TRUNC(DECODE(
                        NVL(PG.IDUTILDTPROC, 'N'),
                        'N',
                        SYSDATE,
                        NVL(PF.DTPROCESSA, SYSDATE)) + NVL(PG.QTDIABLOQCAR, 0)) AND
                    NOT EXISTS (
                        SELECT
                            RE.CDFILIAL
                        FROM
                            RETIRADA RE
                        WHERE
                            RE.CDFILIAL = CA.CDFILIAL AND
                            RE.NRCARDAPIO = CA.NRCARDAPIO
                    ) AND
                    NOT EXISTS (
                        SELECT
                            EF.CDFILIAL
                        FROM
                            EFETCLIE EF
                        WHERE
                            EF.CDFILIAL = CA.CDFILIAL AND
                            EF.CDSERVPLAN = CA.CDSERVFILI AND
                            EF.DTEFETCLIE = CA.DTCARDAPIO AND
                            EF.CDFILIFATU IS NOT NULL AND
                            EF.NRAPURACFAT IS NOT NULL
                    ) AND
                    NOT EXISTS (
                        SELECT
                            DC.NRDISTCARD
                        FROM
                            DISTCARD DC
                        WHERE
                            DC.CDFILIAL = CA.CDFILIAL AND
                            DC.NRCARDAPIO = CA.NRCARDAPIO)";
        $params = array(
            "NRLISTASERV" => $nrlistaserv,
            "DTINICIAL" => $dtini,
            "DTFINAL" => $dtfim
        );
        return $this->connection->fetchAll($query, $params);
    }
    
    private function getCardapioBloqDescFili($dtini, $dtfim, $nrlistaserv) {
        $query = "SELECT
                    CA.CDFILIAL,
                    CA.NRCARDAPIO,
                    CA.CDSERVFILI,
                    SYSDATE,
                    CA.DTCARDAPIO
                FROM
                    CARDADIA CA,
                    LISTA LS
                WHERE
                    LS.NRLISTA =:NRLISTASERV AND
                    CA.CDFILIAL = SUBSTR(LS.CDLISTA, 1, 4) AND
                    CA.CDSERVFILI = SUBSTR(LS.CDLISTA, 5, 5) AND
                    CDDIETPAD IS NULL AND
                    CA.DTCARDAPIO BETWEEN :DTINICIAL AND :DTFINAL
                MINUS (
                    SELECT
                        CA.CDFILIAL,
                        CA.NRCARDAPIO,
                        CA.CDSERVFILI,
                        SYSDATE,
                        CA.DTCARDAPIO
                    FROM
                        CARDADIA CA,
                        PARAMFILIAL PF,
                        PARAMGERAL PG,
                        LISTA LS,
                        CTRLAPROVCARD CT,
                        SERVICO SE
                    WHERE
                        LS.NRLISTA =:NRLISTASERV AND
                        CA.CDFILIAL = SUBSTR(LS.CDLISTA, 1, 4) AND
                        CA.CDSERVFILI = SUBSTR(LS.CDLISTA, 5, 5) AND
                        CDDIETPAD IS NULL AND
                        CA.DTCARDAPIO BETWEEN :DTINICIAL AND :DTFINAL AND
                        PF.CDFILIAL = CA.CDFILIAL AND
                        CT.CDFILIAL (+) = CA.CDFILIAL AND
                        CA.CDFILIAL = SE.CDFILIAL AND
                        CA.CDSERVFILI = SE.CDSERVFILI AND
                        SE.IDCARDDESCENTR = 'S' AND
                        CT.CDSERVFILI (+) = CA.CDSERVFILI AND
                        CT.DTMESCARDAPROV (+) = CA.DTMESCARD AND
                        (CT.DTLIBECARDMATR IS NULL OR 
                        NOT(CT.DTAPROCARDFILI IS NULL)) AND
                        CA.DTCARDAPIO >= DECODE(
                            NVL(PG.IDUTILDTPROC, 'N'),
                            'N',
                            SYSDATE,
                            NVL(PF.DTPROCESSA, SYSDATE)) + NVL(PG.QTDIABLOQCAR, 0) AND
                        NOT EXISTS (
                            SELECT
                                RE.CDFILIAL
                            FROM
                                RETIRADA RE
                            WHERE
                                RE.CDFILIAL = CA.CDFILIAL AND
                                RE.NRCARDAPIO = CA.NRCARDAPIO
                        ) AND
                        NOT EXISTS (
                            SELECT
                                EF.CDFILIAL
                            FROM
                                EFETCLIE EF
                            WHERE
                                EF.CDFILIAL = CA.CDFILIAL AND
                                EF.CDSERVPLAN = CA.CDSERVFILI AND
                                EF.DTEFETCLIE = CA.DTCARDAPIO AND
                                EF.CDFILIFATU IS NOT NULL AND
                                EF.NRAPURACFAT IS NOT NULL
                        ) AND
                        NOT EXISTS (
                            SELECT
                                DC.NRDISTCARD
                            FROM
                                DISTCARD DC
                            WHERE
                                DC.CDFILIAL = CA.CDFILIAL AND
                                DC.NRCARDAPIO = CA.NRCARDAPIO)
                    UNION ALL
                        SELECT
                            CA.CDFILIAL,
                            CA.NRCARDAPIO,
                            CA.CDSERVFILI,
                            SYSDATE,
                            CA.DTCARDAPIO
                        FROM
                            CARDADIA CA,
                            PARAMFILIAL PF,
                            PARAMGERAL PG,
                            LISTA LS,
                            SERVICO SE
                        WHERE
                            LS.NRLISTA =:NRLISTASERV AND
                            CA.CDFILIAL = SUBSTR(LS.CDLISTA, 1, 4) AND
                            CA.CDSERVFILI = SUBSTR(LS.CDLISTA, 5, 5) AND
                            CDDIETPAD IS NULL AND
                            PG.IDPEALTCARDSERV = 'S' AND
                            CA.DTCARDAPIO BETWEEN :DTINICIAL AND :DTFINAL AND
                            CA.CDFILIAL = SE.CDFILIAL AND
                            CA.CDSERVFILI = SE.CDSERVFILI AND
                            NVL(SE.IDCARDDESCENTR, 'N') = 'N' AND
                            PF.CDFILIAL = CA.CDFILIAL AND
                            CA.DTCARDAPIO >= DECODE(
                                NVL(PG.IDUTILDTPROC, 'N'),
                                'N',
                                SYSDATE,
                                NVL(PF.DTPROCESSA, SYSDATE)) + NVL(PG.QTDIABLOQCAR, 0) AND
                            NOT EXISTS (
                                SELECT
                                    RE.CDFILIAL
                                FROM
                                    RETIRADA RE
                                WHERE
                                    RE.CDFILIAL = CA.CDFILIAL AND
                                    RE.NRCARDAPIO = CA.NRCARDAPIO
                            ) AND
                            NOT EXISTS (
                                SELECT
                                    EF.CDFILIAL
                                FROM
                                    EFETCLIE EF
                                WHERE
                                    EF.CDFILIAL = CA.CDFILIAL AND
                                    EF.CDSERVPLAN = CA.CDSERVFILI AND
                                    EF.DTEFETCLIE = CA.DTCARDAPIO AND
                                    EF.CDFILIFATU IS NOT NULL AND
                                    EF.NRAPURACFAT IS NOT NULL
                            ) AND
                            NOT EXISTS (
                                SELECT
                                    DC.NRDISTCARD
                                FROM
                                    DISTCARD DC
                                WHERE
                                    DC.CDFILIAL = CA.CDFILIAL AND
                                    DC.NRCARDAPIO = CA.NRCARDAPIO)
                )";
        $params = array(
            "NRLISTASERV" => $nrlistaserv,
            "DTINICIAL" => $dtini,
            "DTFINAL" => $dtfim
        );
        return $this->connection->fetchAll($query, $params);
    }
    
    private function getCardapioPadrMatr($dtini, $dtfim, $nrlistaserv) {
        $query = "SELECT
                    CA.CDFILIAL,
                    CA.NRCARDAPIO,
                    CA.CDSERVFILI,
                    SYSDATE,
                    CA.DTCARDAPIO
                FROM
                    CARDADIA CA,
                    PARAMFILIAL PF,
                    PARAMGERAL PG,
                    CARDAMES CM,
                    LISTA LS
                WHERE
                    LS.NRLISTA =:NRLISTASERV AND
                    CA.CDFILIAL = SUBSTR(LS.CDLISTA, 1, 4) AND
                    CA.CDSERVFILI = SUBSTR(LS.CDLISTA, 5, 5) AND
                    CDDIETPAD IS NULL AND
                    CA.DTCARDAPIO BETWEEN :DTINICIAL AND :DTFINAL AND
                    PF.CDFILIAL = CA.CDFILIAL AND
                    CA.CDFILIAL = CM.CDFILIAL (+) AND
                    CA.CDSERVFILI = CM.CDSERVFILI (+) AND
                    CA.DTMESCARD = CM.DTMESCARDAPIO (+) AND
                    (
                        CM.IDCARDLIBAGRE <> 'S' OR
                        CM.IDCARDLIBAGRE IS NULL
                    ) AND
                    CA.DTCARDAPIO >= TRUNC(DECODE(
                        NVL(PG.IDUTILDTPROC, 'N'),
                        'N',
                        SYSDATE,
                        NVL(PF.DTPROCESSA, SYSDATE)) + NVL(PG.QTDIABLOQCAR, 0)) AND
                    NOT EXISTS (
                        SELECT
                            RE.CDFILIAL
                        FROM
                            RETIRADA RE
                        WHERE
                            RE.CDFILIAL = CA.CDFILIAL AND
                            RE.NRCARDAPIO = CA.NRCARDAPIO
                    ) AND
                    NOT EXISTS (
                        SELECT
                            EF.CDFILIAL
                        FROM
                            EFETCLIE EF
                        WHERE
                            EF.CDFILIAL = CA.CDFILIAL AND
                            EF.CDSERVPLAN = CA.CDSERVFILI AND
                            EF.DTEFETCLIE = CA.DTCARDAPIO AND
                            EF.CDFILIFATU IS NOT NULL AND
                            EF.NRAPURACFAT IS NOT NULL
                    ) AND
                    NOT EXISTS (
                        SELECT
                            DC.NRDISTCARD
                        FROM
                            DISTCARD DC
                        WHERE
                            DC.CDFILIAL = CA.CDFILIAL AND
                            DC.NRCARDAPIO = CA.NRCARDAPIO)";
        $params = array(
            "NRLISTASERV" => $nrlistaserv,
            "DTINICIAL" => $dtini,
            "DTFINAL" => $dtfim
        );
        return $this->connection->fetchAll($query, $params);
    }
    
    private function getCardapioBloqPadrMatr($dtini, $dtfim, $nrlistaserv) {
        $query = "SELECT
                    CA.CDFILIAL,
                    CA.NRCARDAPIO,
                    CA.CDSERVFILI,
                    SYSDATE,
                    CA.DTCARDAPIO
                FROM
                    CARDADIA CA,
                    LISTA LS
                WHERE
                    LS.NRLISTA =:NRLISTASERV AND
                    CA.CDFILIAL = SUBSTR(LS.CDLISTA, 1, 4) AND
                    CA.CDSERVFILI = SUBSTR(LS.CDLISTA, 5, 5) AND
                    CDDIETPAD IS NULL AND
                    CA.DTCARDAPIO BETWEEN :DTINICIAL AND :DTFINAL
                MINUS
                SELECT
                    CA.CDFILIAL,
                    CA.NRCARDAPIO,
                    CA.CDSERVFILI,
                    SYSDATE,
                    CA.DTCARDAPIO
                FROM
                    CARDADIA CA,
                    PARAMFILIAL PF,
                    PARAMGERAL PG,
                    CARDAMES CM,
                    LISTA LS
                WHERE
                    LS.NRLISTA =:NRLISTASERV AND
                    CA.CDFILIAL = SUBSTR(LS.CDLISTA, 1, 4) AND
                    CA.CDSERVFILI = SUBSTR(LS.CDLISTA, 5, 5) AND
                    CDDIETPAD IS NULL AND
                    CA.DTCARDAPIO BETWEEN :DTINICIAL AND :DTFINAL AND
                    PF.CDFILIAL = CA.CDFILIAL AND
                    CA.CDFILIAL = CM.CDFILIAL (+) AND
                    CA.CDSERVFILI = CM.CDSERVFILI (+) AND
                    CA.DTMESCARD = CM.DTMESCARDAPIO (+) AND
                    (
                        CM.IDCARDLIBAGRE <> 'S' OR
                        CM.IDCARDLIBAGRE IS NULL
                    ) AND
                    CA.DTCARDAPIO >= DECODE(
                        NVL(PG.IDUTILDTPROC, 'N'),
                        'N',
                        SYSDATE,
                        NVL(PF.DTPROCESSA, SYSDATE)) + NVL(PG.QTDIABLOQCAR, 0) AND
                    NOT EXISTS (
                        SELECT
                            RE.CDFILIAL
                        FROM
                            RETIRADA RE
                        WHERE
                            RE.CDFILIAL = CA.CDFILIAL AND
                            RE.NRCARDAPIO = CA.NRCARDAPIO
                    ) AND
                    NOT EXISTS (
                        SELECT
                            EF.CDFILIAL
                        FROM
                            EFETCLIE EF
                        WHERE
                            EF.CDFILIAL = CA.CDFILIAL AND
                            EF.CDSERVPLAN = CA.CDSERVFILI AND
                            EF.DTEFETCLIE = CA.DTCARDAPIO AND
                            EF.CDFILIFATU IS NOT NULL AND
                            EF.NRAPURACFAT IS NOT NULL
                    ) AND
                    NOT EXISTS (
                        SELECT
                            DC.NRDISTCARD
                        FROM
                            DISTCARD DC
                        WHERE
                            DC.CDFILIAL = CA.CDFILIAL AND
                            DC.NRCARDAPIO = CA.NRCARDAPIO)";
        $params = array(
            "NRLISTASERV" => $nrlistaserv,
            "DTINICIAL" => $dtini,
            "DTFINAL" => $dtfim
        );
        return $this->connection->fetchAll($query, $params);
    }
    
    private function getCardapioPadrFili($dtini, $dtfim, $nrlistaserv) {
        $query = "SELECT
                    CA.CDFILIAL,
                    CA.NRCARDAPIO,
                    CA.CDSERVFILI,
                    SYSDATE,
                    CA.DTCARDAPIO
                FROM
                    CARDADIA CA,
                    PARAMFILIAL PF,
                    PARAMGERAL PG,
                    CARDAMES CMC,
                    CARDAMES CMF,
                    FILIAL F,
                    LISTA LS
                WHERE
                    LS.NRLISTA =:NRLISTASERV AND
                    CA.CDFILIAL = SUBSTR(LS.CDLISTA, 1, 4) AND
                    CA.CDSERVFILI = SUBSTR(LS.CDLISTA, 5, 5) AND
                    CDDIETPAD IS NULL AND
                    CA.DTCARDAPIO BETWEEN :DTINICIAL AND :DTFINAL AND
                    F.CDFILIAL = CA.CDFILIAL AND
                    PF.CDFILIAL = CA.CDFILIAL AND
                    F.CDFILCENTPME = CMC.CDFILIAL AND
                    CA.CDSERVFILI = CMC.CDSERVFILI AND
                    CA.DTMESCARD = CMC.DTMESCARDAPIO AND
                    CMC.IDCARDLIBCENT = 'S' AND
                    CA.CDFILIAL = CMF.CDFILIAL (+) AND
                    CA.CDSERVFILI = CMF.CDSERVFILI (+) AND
                    CA.DTMESCARD = CMF.DTMESCARDAPIO (+) AND
                    (
                        CMF.IDCARDLIBAGRE <> 'S' OR
                        CMF.IDCARDLIBAGRE IS NULL
                    ) AND
                    CA.DTCARDAPIO >= TRUNC(DECODE(
                        NVL(PG.IDUTILDTPROC, 'N'),
                        'N',
                        SYSDATE,
                        NVL(PF.DTPROCESSA, SYSDATE)) + NVL(PG.QTDIABLOQCAR, 0)) AND
                    NOT EXISTS (
                        SELECT
                            RE.CDFILIAL
                        FROM
                            RETIRADA RE
                        WHERE
                            RE.CDFILIAL = CA.CDFILIAL AND
                            RE.NRCARDAPIO = CA.NRCARDAPIO
                    ) AND
                    NOT EXISTS (
                        SELECT
                            EF.CDFILIAL
                        FROM
                            EFETCLIE EF
                        WHERE
                            EF.CDFILIAL = CA.CDFILIAL AND
                            EF.CDSERVPLAN = CA.CDSERVFILI AND
                            EF.DTEFETCLIE = CA.DTCARDAPIO AND
                            EF.CDFILIFATU IS NOT NULL AND
                            EF.NRAPURACFAT IS NOT NULL
                    ) AND
                    NOT EXISTS (
                        SELECT
                            DC.NRDISTCARD
                        FROM
                            DISTCARD DC
                        WHERE
                            DC.CDFILIAL = CA.CDFILIAL AND
                            DC.NRCARDAPIO = CA.NRCARDAPIO)";
        $params = array(
            "NRLISTASERV" => $nrlistaserv,
            "DTINICIAL" => $dtini,
            "DTFINAL" => $dtfim
        );
        return $this->connection->fetchAll($query, $params);
    }
    
    private function getCardapioBloqPadrFili($dtini, $dtfim, $nrlistaserv) {
        $query = "SELECT
                    CA.CDFILIAL,
                    CA.NRCARDAPIO,
                    CA.CDSERVFILI,
                    SYSDATE,
                    CA.DTCARDAPIO
                FROM
                    CARDADIA CA,
                    LISTA LS
                WHERE
                    LS.NRLISTA =:NRLISTASERV AND
                    CA.CDFILIAL = SUBSTR(LS.CDLISTA, 1, 4) AND
                    CA.CDSERVFILI = SUBSTR(LS.CDLISTA, 5, 5) AND
                    CDDIETPAD IS NULL AND
                    CA.DTCARDAPIO BETWEEN :DTINICIAL AND :DTFINAL
                MINUS
                SELECT
                    CA.CDFILIAL,
                    CA.NRCARDAPIO,
                    CA.CDSERVFILI,
                    SYSDATE,
                    CA.DTCARDAPIO
                FROM
                    CARDADIA CA,
                    PARAMFILIAL PF,
                    PARAMGERAL PG,
                    CARDAMES CMC,
                    CARDAMES CMF,
                    FILIAL F,
                    LISTA LS
                WHERE
                    LS.NRLISTA =:NRLISTASERV AND
                    CA.CDFILIAL = SUBSTR(LS.CDLISTA, 1, 4) AND
                    CA.CDSERVFILI = SUBSTR(LS.CDLISTA, 5, 5) AND
                    CDDIETPAD IS NULL AND
                    CA.DTCARDAPIO BETWEEN :DTINICIAL AND :DTFINAL AND
                    F.CDFILIAL = CA.CDFILIAL AND
                    PF.CDFILIAL = CA.CDFILIAL AND
                    F.CDFILCENTPME = CMC.CDFILIAL AND
                    CA.CDSERVFILI = CMC.CDSERVFILI AND
                    CA.DTMESCARD = CMC.DTMESCARDAPIO AND
                    CMC.IDCARDLIBCENT = 'S' AND
                    CA.CDFILIAL = CMF.CDFILIAL (+) AND
                    CA.CDSERVFILI = CMF.CDSERVFILI (+) AND
                    CA.DTMESCARD = CMF.DTMESCARDAPIO (+) AND
                    (
                        CMF.IDCARDLIBAGRE <> 'S' OR
                        CMF.IDCARDLIBAGRE IS NULL
                    ) AND
                    CA.DTCARDAPIO >= DECODE(
                        NVL(PG.IDUTILDTPROC, 'N'),
                        'N',
                        SYSDATE,
                        NVL(PF.DTPROCESSA, SYSDATE)) + NVL(PG.QTDIABLOQCAR, 0) AND
                    NOT EXISTS (
                        SELECT
                            RE.CDFILIAL
                        FROM
                            RETIRADA RE
                        WHERE
                            RE.CDFILIAL = CA.CDFILIAL AND
                            RE.NRCARDAPIO = CA.NRCARDAPIO
                    ) AND
                    NOT EXISTS (
                        SELECT
                            EF.CDFILIAL
                        FROM
                            EFETCLIE EF
                        WHERE
                            EF.CDFILIAL = CA.CDFILIAL AND
                            EF.CDSERVPLAN = CA.CDSERVFILI AND
                            EF.DTEFETCLIE = CA.DTCARDAPIO AND
                            EF.CDFILIFATU IS NOT NULL AND
                            EF.NRAPURACFAT IS NOT NULL
                    ) AND
                    NOT EXISTS (
                        SELECT
                            DC.NRDISTCARD
                        FROM
                            DISTCARD DC
                        WHERE
                            DC.CDFILIAL = CA.CDFILIAL AND
                            DC.NRCARDAPIO = CA.NRCARDAPIO)";
        $params = array(
            "NRLISTASERV" => $nrlistaserv,
            "DTINICIAL" => $dtini,
            "DTFINAL" => $dtfim
        );
        return $this->connection->fetchAll($query, $params);
    }
    
    public function returnServFili($servico){
        $servFili = array();
        foreach ($servico as $key => &$value){
            $servFili[] = $value['CDFILIAL'].$value['CDSERVFILI'];
        }
        return $servFili;
    }
    
    public function verificaEfetivos($request, $response) {
        $row = $request->getRow();
        $dtini = isset($row['DTINTERVAL']['start']) ? $row['DTINTERVAL']['start'] : $row['dataInicial'] ;
        $dtfim = isset($row['DTINTERVAL']['end']) ? $row['DTINTERVAL']['end'] : $row['dataFinal'] ;
      
        if($row['ALLCDSERVFILI'] === 'S') {
            $servicos = $this->getServicos($row['CDGRUPFILI'] ? $row['CDGRUPFILI'] : 'T', $row['ALLCDFILIAL'] === 'S' ? 'T' : join('_', $row['CDFILIAL']));
            // $nrlistaserv = $this->insertListaServicos($servicos);
            $servFili = self::returnServFili($servicos);
            $nrlistaserv = $this->util->insertLista($servFili);
        } else {
            // $nrlistaserv = $this->insertListaServicos($row['FILISERV']);
            $nrlistaserv = $this->util->insertLista($row['FILISERV']);
        }
        
        $sql = "SELECT DISTINCT S.CDFILIAL, F.NMFILIAL, S.CDSERVFILI, S.NMSERVICO, E.DTEFETCLIE, E.NRAPURACFAT
                  FROM EFETCLIE E
                       LEFT JOIN FILIAL F ON F.CDFILIAL = E.CDFILIAL
                       LEFT JOIN SERVICO S ON S.CDFILIAL = E.CDFILIAL AND
                                              S.CDSERVFILI = E.CDSERVPLAN
                       INNER JOIN LISTA L ON L.NRLISTA = :NRLISTA AND
                                             E.CDFILIAL = SUBSTR(L.CDLISTA, 1, 4) AND
                                             E.CDSERVPLAN = SUBSTR(L.CDLISTA, 5, 5)
                 WHERE L.NRLISTA = :NRLISTA
                   AND E.DTEFETCLIE BETWEEN :DTINI AND :DTFIN";
                   
        $params = array(
            "NRLISTA" => $nrlistaserv,
            "DTINI" => $dtini,
            "DTFIN" => $dtfim,
        );
        $result = $this->connection->fetchAll($sql, $params);
        $response->addDataSet(new DataSet('result', $result));
    }
}